@@ -15,6 +15,7 @@ import conduit
 import conduit.ModuleWrapper as ModuleWrapper
 import conduit.utils as Utils
 import conduit.Settings as Settings
+import conduit.XMLSerialization as XMLSerialization
 
 STATUS_NONE = _("Ready")
 STATUS_CHANGE_DETECTED = _("New data to sync")
@@ -27,7 +28,7 @@ STATUS_DONE_SYNC_ERROR = _("Error Synchronizing")
 STATUS_DONE_SYNC_SKIPPED = _("Synchronization Skipped")
 STATUS_DONE_SYNC_CANCELLED = _("Synchronization Cancelled")
 STATUS_DONE_SYNC_CONFLICT = _("Synchronization Conflict")
-STATUS_DONE_SYNC_NOT_CONFIGURED = _("Not Configured Correctly")
+STATUS_DONE_SYNC_NOT_CONFIGURED = _("Not Configured")
 
 class DataProviderBase(gobject.GObject):
     """
@@ -57,6 +58,8 @@ class DataProviderBase(gobject.GObject):
         self.pendingChangeDetected = False
         self.icon = None
         self.status = STATUS_NONE
+        self.config_container = None
+        self.configuration = {}
 
     def __emit_status_changed(self):
         """
@@ -168,13 +171,69 @@ class DataProviderBase(gobject.GObject):
         else:
             return False
 
-    def configure(self, window):
+    def get_config_container(self, configContainerKlass, name, icon, configurator):
         """
-        Show a configuration box for configuring the dataprovider instance.
-        @param window: The parent gtk.Window (to show a modal dialog)
+        Retrieves the configuration container
+        @param configContainerKlass: The class used to instantiate the graphical
+        configuration widget.
+        @param name: The name of the dataprovider being configured. Typically
+        used in the graphical config widget
+        @param icon: The icon of the dataprovider being configured. Typically
+        used in the graphical config widget
+        @param configurator: The configurator object
         """
+        # If the dataprovider is using the old system, returns None (a message
+        # will be thrown in the Canvas module)
+        if hasattr(self, "configure"):
+            return None
+        if not self.config_container:
+            self.config_container = configContainerKlass(self, configurator)
+            self.config_container.name = name
+            self.config_container.icon = icon
+            self.config_container.connect('apply', self.config_apply)
+            self.config_container.connect('cancel', self.config_cancel)
+            self.config_container.connect('show', self.config_show)
+            self.config_container.connect('hide', self.config_hide)
+            self.config_setup(self.config_container)
+            # This is definetely just for debugging (it prints everything
+            # that is changed in the configuration dialog)
+            #self.config_container.connect(
+            #        "item-changed", 
+            #        lambda c, i: log.debug("%s: %s = %s" % (i.title, i.config_name, i.get_value()))
+            #)
+        return self.config_container
+    
+    def config_setup(self, config_container):
+        '''
+        Called when the configuration container was just built. Should be 
+        implemented by subclasses that want to show their own configuration.
+        '''
         pass
-
+    
+    def config_show(self, config_container):
+        '''
+        Called when the configuration is about to be shown
+        '''
+        pass
+    
+    def config_hide(self, config_container):
+        '''
+        Called when the configuration is about to be hidden
+        '''
+        pass
+        
+    def config_apply(self, config_container):
+        '''
+        Called when the configuration was applied
+        '''
+        pass
+        
+    def config_cancel(self, config_container):
+        '''
+        Called when the configuration was cancelled
+        '''
+        pass
+        
     def is_configured(self, isSource, isTwoWay):
         """
         Checks if the dp has been configured or not (and if it needs to be)
@@ -191,6 +250,19 @@ class DataProviderBase(gobject.GObject):
         @returns: Dictionary of strings containing application settings
         @rtype: C{dict(string)}
         """
+        if self.configuration:
+            ret = {}
+            for name, default, setter, getter in self._get_configuration_parameters(self.configuration):
+                if getter:
+                    value = getter()
+                elif hasattr(self, name):
+                    value = getattr(self, name)
+                else:
+                    value = default
+                ret[name] = value
+            return ret
+        else:
+            log.critical("No configuration set (probably old-style module)")
         return {}
 
     def get_configuration_xml(self):
@@ -198,76 +270,117 @@ class DataProviderBase(gobject.GObject):
         Returns the dataprovider configuration as xml
         @rtype: C{string}
         """
-        doc = xml.dom.minidom.Element("configuration")
+        xml_configuration = XMLSerialization.Settings()
         configDict = self.get_configuration()
-        for config in configDict:
-                configxml = xml.dom.minidom.Element(str(config))
-                #store the value and value type
-                try:
-                    vtype = Settings.TYPE_TO_TYPE_NAME[ type(configDict[config]) ]
-                    value = Settings.TYPE_TO_STRING[  type(configDict[config]) ](configDict[config])
-                except KeyError:
-                    log.warn("Cannot convert %s to string. Value of %s not saved" % (type(value), config))
-                    vtype = Settings.TYPE_TO_TYPE_NAME[str]
-                    value = Settings.TYPE_TO_STRING[str](configDict[config])
-                configxml.setAttribute("type", vtype)
-                valueNode = xml.dom.minidom.Text()
-                valueNode.data = value
-                configxml.appendChild(valueNode)
-                doc.appendChild(configxml)
+        for name, value in configDict.iteritems():
+            xml_configuration[name] = value
+        return xml_configuration.xml_document.toxml()
 
-        return doc.toxml()
+    def _get_configuration_parameters(self, configuration):
+        '''
+        Normalize the configuration dict to 3 params and yields the name plus
+        the parameters. See update_configuration for more information.
+        '''
+        for name, params in configuration.iteritems():
+            if not isinstance(params, tuple):
+                params = (params,)
+            # Normalize to 3 parameters plus name
+            yield (name,) + params + (None,) * (3 - len(params))
+
+    def _set_configuration_values(self, configuration, config = None):
+        '''
+        Set attributes according to configuration. See update_configuration
+        for more information.
+        '''
+        for name, default, setter, getter in self._get_configuration_parameters(configuration):
+            if hasattr(self, name) and callable(getattr(self, name)):
+                continue            
+            if not config or (name in config):
+                klass = None
+                if default is not None:            
+                    klass = default.__class__
+                if config:
+                    value = config[name]
+                    #FIXME: Wrap a try/except clause with logging
+                    if klass:
+                        value = klass(value)
+                else:
+                    value = default
+                if setter:
+                    if not hasattr(self, name):
+                        setattr(self, name, value)
+                    setter(value)
+                else:
+                    setattr(self, name, value)        
+
+    def update_configuration(self, **kwargs):
+        '''
+        Set the configuration values to be automatically saved and loaded.
+        
+        The keys to kwargs are the attribute names that will be used. The values
+        to kwargs may be a default value or a tuple, containing the default
+        value, a setter and a getter. Not all values must exist in the 
+        tuple. In the case the tuple's length is smaller then 3, the later 
+        properties are defaulted to None.
+        The default value is immediately applied to the attribute if no other 
+        value is set to that attribute. So calling this function on 
+        initialization already initializes all attributes.
+        The getter and setter are functions to get and set the value. They are
+        very simple, getter should return the value to an attribute, so that
+        value can be saved, while setter, which receives a value trough it's 
+        arguments, should propably set an attribute with that value.
+        Notice that if the setter is used, the value is not automatically set
+        as an attribute, unless that attribute does not exist. This allows a 
+        setter to compare the current attribute value to new value to be set,
+        and only set the new value if it wishes so.
+        
+        Note if the dataprovider overrides set_configuration or 
+        get_configuration without calling the implementations in this class, 
+        then the properties defined here have no affect. Either do not override 
+        those functions, or call them like 
+        DataProviderBase.get_configuration(self).
+        '''
+        #FIXME: Rewrite and clarify the documentation above
+
+        self.configuration.update(kwargs)
+        self._set_configuration_values(kwargs)
 
     def set_configuration(self, config):
         """
         Restores applications settings
         @param config: dictionary of dataprovider settings to restore
         """
-        for c in config:
-            #Perform these checks to stop malformed xml from stomping on
-            #unintended variables or posing a security risk by overwriting methods
-            if getattr(self, c, None) != None and callable(getattr(self, c, None)) == False:
-                setattr(self,c,config[c])
-            else:
-                log.warn("Not restoring %s setting: Exists=%s Callable=%s" % (
-                    c,
-                    getattr(self, c, False),
-                    callable(getattr(self, c, None)))
-                    )
+        if self.configuration:
+            self._set_configuration_values(self.configuration, config)
+        else:
+            for c in config:
+                #Perform these checks to stop malformed xml from stomping on
+                #unintended variables or posing a security risk by overwriting methods
+                if getattr(self, c, None) != None and callable(getattr(self, c, None)) == False:
+                    setattr(self,c,config[c])
+                else:
+                    log.warn("Not restoring %s setting: Exists=%s Callable=%s" % (
+                        c,
+                        getattr(self, c, False),
+                        callable(getattr(self, c, None)))
+                        )
 
-    def set_configuration_xml(self, xmltext):
+    def set_configuration_xml(self, xmltext, xmlversion):
         """
         Restores applications settings from XML
 
         @param xmltext: xml representation of settings
         @type xmltext: C{string}
         """
-        doc = xml.dom.minidom.parseString(xmltext)
-        configxml = doc.documentElement
-
-        if configxml.nodeType == configxml.ELEMENT_NODE and configxml.localName == "configuration":
-            settings = {}
-            for s in configxml.childNodes:
-                if s.nodeType == s.ELEMENT_NODE:
-                    #now convert the setting to the correct type (if filled out)
-                    if s.hasChildNodes():
-                        raw = s.firstChild.data
-                        vtype = s.getAttribute("type")
-                        try:
-                            data = Settings.STRING_TO_TYPE[vtype](raw)
-                        except KeyError:
-                            #fallback to string type
-                            log.warn("Cannot convert string (%s) to native type %s\n" % (raw, vtype, traceback.format_exc()))
-                            data = str(raw)
-                        settings[s.localName] = data
-
-            try:
-                self.set_configuration(settings)
-            except Exception, err: 
-                log.warn("Error restoring %s configuration\n%s" % 
-                        (self._name_, traceback.format_exc()))
-        else:
-            log.debug("Could not find <configuration> xml fragment")
+        xml_configuration = XMLSerialization.Settings(xmltext, xmlversion)
+        settings = {}
+        for name, value in xml_configuration:
+            settings[name] = value
+        try:
+            self.set_configuration(settings)
+        except Exception, err: 
+            log.warn("Error restoring %s configuration\n%s" % 
+                    (self._name_, traceback.format_exc()))
 
     def get_UID(self):
         """
